# Copyright (c) 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from magnumclient.v1 import client


def Client(version='1', **kwargs):
    """Factory function to create a new container service client."""
    if version != '1':
        raise ValueError(
            "magnum only has one API version. Valid values for 'version'"
            " are '1'")
    return client.Client(**kwargs)
